*==============================================================================
* Program:			CheckUpdate
* Purpose:			Compares files on a server and a local workstation: if the
*						server has newer versions, the new versions are copied
*						to the workstation and an external EXE used to unzip
*						ZIP files
* Author:			Doug Hennig
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Last Revision:	04/10/2001
* Parameters:		none
* Environment in:	CONFIG.DBF must exist and contain the following fields:
*						UpdateFile: the name of the text file containing a
*							list of files to update from server copies. Note:
*							the first line of this file could be "VERSION="
*							followed by a version number to compare to the
*							current version of the application
*						Quiet: .T. to not display any progress messages
*						Prompt: .T. to prompt if the user wants to download
*							the update
*						ServerName: the name of the server where application
*							files are found. Note: this can be left blank for
*							LAN/WAN updates; it's only required for Internet
*							updates
*						ServerPath: the server directory where application
*							 files are found
*						UserName: the user name for the server. Note: this can
*							be left blank for LAN/WAN updates; it's only
*							required for Internet updates
*						Password: the password for the server. Note: this can
*							be left blank for LAN/WAN updates; it's only
*							required for Internet updates
*						Title: the title for MESSAGEBOX dialogs
*						AppName: the name of the EXE to unzip files
*						UpdClass: the update class to instantiate
*						UpdLibrary: the library the class specified in UpdClass
*							is contained in
* Environment out:	the files specified in the update file may have been copied
*						from the server to the workstation
*					if everything is OK, the application specified in AppName
*						was executed and this application terminated
*==============================================================================

local lcConfig, ;
	lcDirectory, ;
	UpdateFile, ;
	Quiet, ;
	Prompt, ;
	ServerName, ;
	ServerPath, ;
	UserName, ;
	Password, ;
	Title, ;
	AppName, ;
	UpdClass, ;
	UpdLibrary, ;
	lcCurrEXE, ;
	llReturn, ;
	loUpdate, ;
	laFiles[1], ;
	lnFiles, ;
	llVersion, ;
	lcVersion, ;
	laVersion[1], ;
	lnItems, ;
	lnPos1, ;
	lnPos2, ;
	lcCurrVersion, ;
	lcFiles, ;
	lnI, ;
	lcFile, ;
	lcUpdateFile, ;
	lcCaption
#include FOXPRO.H

* Try to find the configuration table.

lcConfig = 'CONFIG.DBF'
if not file(lcConfig)
	lcDirectory = justpath(sys(16))
	lcConfig    = lcDirectory + lcConfig
	if not file(lcConfig)
		return .F.
	endif not file(lcConfig)
endif not file(lcConfig)

* Open the configuration table and get the information we need.

select 0
use (lcConfig) again shared
scatter memvar memo
use
UpdateFile = trim(m.UpdateFile)
ServerName = trim(m.ServerName)
ServerPath = trim(m.ServerPath)
UserName   = trim(m.UserName)
Password   = trim(m.Password)
Title      = trim(m.Title)
AppName    = trim(m.AppName)
UpdClass   = trim(m.UpdClass)
UpdLibrary = trim(m.UpdLibrary)

* Get the name of the current EXE.

lcCurrEXE = sys(16, 1)

* Instantiate the updater object and have it get the most recent list of files
* to update.

loUpdate = newobject(m.UpdClass, m.UpdLibrary)
with loUpdate
	dimension .aFiles[1]
	.aFiles[1]          = m.UpdateFile
	.cServerName        = m.ServerName
	.cServerDirectory   = m.ServerPath
	.cUserName          = m.UserName
	.cPassword          = m.Password
	.cTitle             = m.Title
	.lPromptForDownload = .F.
	.lCheckVersion      = .F.
	.lQuiet             = .T.
	llReturn            = .UpdateLocal()
	do case

* If we received the update file list, check if the first line of the file is
* "VERSION=". If so, get the version number from the rest of the line. Then
* delete the file.

		case llReturn
			lnFiles = alines(laFiles, filetostr(m.UpdateFile))
			erase (m.UpdateFile)
			if upper(laFiles[1]) = 'VERSION='
				lcVersion = upper(alltrim(substr(laFiles[1], 9)))
				adel(laFiles, 1)
				lnFiles = lnFiles - 1
				dimension laFiles[lnFiles]
				llVersion = .T.
			endif upper(laFiles[1]) = 'VERSION='

* Get the current version.

			lnItems = agetfileversion(laVersion, lcCurrEXE)
			if lnItems >= 4
				lnPos1 = at('.', laVersion[4])
				lnPos2 = at('.', laVersion[4], 2)
				lcCurrVersion = left(laVersion[4], lnPos1) + ;
					padl(substr(laVersion[4], lnPos1 + 1, ;
						lnPos2 - lnPos1 - 1), 2, '0')
			endif lnItems >= 4

* If the server version number is newer than the current one, we'll do the
* process. If we didn't find server or application version numbers, we'll do
* the process anyway.

			if not llVersion or empty(lcVersion) or empty(lcCurrVersion) or ;
				lcVersion > lcCurrVersion
				dimension .aFiles[lnFiles]
				acopy(laFiles, .aFiles)
				.lQuiet             = m.Quiet
				.lPromptForDownload = m.Prompt
				.lCheckVersion      = not llVersion or empty(lcCurrVersion)
				llReturn            = .UpdateLocal()
				do case

* If we successfully downloaded the update files, unzip any ZIP files. We have
* to run an external program to do this, since presumably the ZIP file would
* contain a newer version of this EXE.

					case llReturn
						lcFiles = ''
						for lnI = 1 to lnFiles
							lcFile = laFiles[lnI]
							if upper(justext(lcFile)) = 'ZIP'
								lcFiles = lcFiles + laFiles[lnI] + chr(13)
							endif upper(justext(lcFile)) = 'ZIP'
						next lnI
						lcUpdateFile = sys(2015) + '.txt'
						strtofile(lcFiles, lcUpdateFile)
						lcCaption = _screen.Caption
						run /n1 "&AppName" "&lcCaption" "&lcUpdateFile" ;
							"&lcCurrEXE"
						on shutdown
						quit

* Display an error message if a problem occurred.

					case not empty(.cErrorMessage) and ;
						messagebox(m.Title + ' could not be updated at ' + ;
						'this time. Do you want to see details?', ;
						MB_YESNO + MB_ICONEXCLAMATION, ;
						m.Title + ' Internet Update Error') = IDYES
						messagebox('Details:' + chr(13) + chr(13) + ;
							.cErrorMessage, MB_OK + MB_ICONEXCLAMATION, ;
							m.Title + ' Internet Update Error')
				endcase
			endif not llVersion ...

* We couldn't receive the update file list, so display an error message.

		case not empty(.cErrorMessage) and ;
			messagebox('I cannot check if a newer version of ' + m.Title + ;
			' is available at this time. Do you want to see details?', ;
			MB_YESNO + MB_ICONEXCLAMATION, ;
			m.Title + ' Internet Update Error') = IDYES
			messagebox('Details:' + chr(13) + chr(13) + ;
				.cErrorMessage, MB_OK + MB_ICONEXCLAMATION, ;
				m.Title + ' Internet Update Error')

* The update file list didn't exist, so display a message.

		case empty(.cErrorMessage) and not file(m.UpdateFile)
			messagebox('You are running the most recent version.', ;
				MB_OK + MB_ICONINFORMATION, m.Title + ' Internet Update')
	endcase
endwith
return llReturn
